Crate midnight_storage

Crate midnight_storage 

Modules

arena
An Arena for storing Merkle-ized data structures in memory, persisting them to disk, and reloading them from disk.
backend
A storage layer that intermediates between the in-memory Arena and the persistent database, managing reference counts for the Merkle-ized DAGs we store in the DB, and providing a caching layer.
db
A database of content-addressed DAG nodes.
delta_tracking
Delta tracking module for write and delete costing
merkle_patricia_trie
Merkle Patricia Tries.
storable
A trait defining a Storable object, which can be assembled into a tree.
storage
storage containers

Structs

Storage
A factory for various storage objects

Traits

Storable
A Storable object.
WellBehavedHasher
Super-trait containing all requirements for a Hasher

Type Aliases

DefaultDB
The default database.
DefaultHasher
The default storage mechanism.

Derive Macros

Storable
#[derive(Storable)] macro.