Sp

Struct Sp 

pub struct Sp<T, D = InMemoryDB>
where T: 'static + ?Sized, D: DB,
{ pub child_repr: ArenaKey<<D as DB>::Hasher>, pub arena: Arena<D>, pub root: ArenaHash<<D as DB>::Hasher>, /* private fields */ }
A typed pointer to a value stored in the Arena.

An Sp<T> can be lazily initialized, in which case its internal T value won’t be loaded until access is attempted.

Fields

child_repr: ArenaKey<<D as DB>::Hasher>

This Sp represented as a child node (for easy access)

arena: Arena<D>

The arena this Sp points into

root: ArenaHash<<D as DB>::Hasher>

The persistent hash of data.

Implementations

Source

impl<T, D> Sp<T, D>
where T: Storable<D>, D: DB,

pub fn new(value: T) -> Sp<T, D>

Allocates a new Sp against the default storage

Source

impl<T, D> Sp<T, D>
where T: 'static + ?Sized, D: DB,

Constructors for Sp that take care of Arena ref counting.

pub fn into_tracked(&self) -> Sp<T, D>

Converts this Sp into one that is tracked directly, making it possible to lookup by its hash.

This forces the Sp to be considered a reference when used as a child of other Sps, and places it into internal caches, and eventually the database if persisted or a parent is persisted.

Source

impl<D> Sp<dyn Any + Send + Sync, D>
where D: DB,

pub fn downcast<T>(&self) -> Option<Sp<T, D>>
where T: Any + Send + Sync,

Downcasts this dynamically typed pointer to a concrete type, if possible.

pub fn force_downcast<T>(&self) -> Sp<T, D>
where T: Any + Send + Sync,

Downcasts this dynamically typed pointer to a concrete type, but pushes through the cast regardless of the underlying type.

This will effectively unload the Sp, and construct a new lazy Sp with the same backing data. There is no way of knowing if this will succeed, as the lazy loading will defer failure to a context where a failure panics.

Source

impl<T, D> Sp<T, D>
where T: Any + Send + Sync, D: DB,

pub fn upcast(&self) -> Sp<dyn Any + Send + Sync, D>

Casts this pointer into a dynamically typed Any pointer.

Source

impl<T, D> Sp<T, D>
where D: DB, T: ?Sized,

pub fn is_lazy(&self) -> bool

Return true iff this Sp is lazy/unforced, i.e. its data has not yet been loaded.

If the Sp is lazy, it can be forced by dereferencing it.

pub fn hash(&self) -> ArenaHash<<D as DB>::Hasher>

Return hash of self and all children, cached from &lt;T as Storable&gt;::hash().

This is the root key of self, as a content-addressed Merkle node.

pub fn as_typed_key(&self) -> TypedArenaKey<T, <D as DB>::Hasher>

Returns the TypedArenaKey representation of this Sp, useful as a reference to persist.

pub fn as_child(&self) -> ArenaKey<<D as DB>::Hasher>

Returns the ArenaKey representation of this Sp, being either a ref to [Sp::hash], or the direct encoding for small children.

Source

impl<T, D> Sp<T, D>
where T: Storable<D>, D: DB,

pub fn persist(&mut self)

Notify the storage back-end to increment the persist count on this object.

See [StorageBackend::persist].

Note: Due to a technicality in past behaviour, this converts the Sp into a Ref internally. In particular, this means that Sp::as_typed_key will provide different results before and after persisting. A change to this would be a major breaking change, and is therefore deferred.

pub fn unpersist(&self)

Notify the storage back-end to decrement the persist count on this object.

See [StorageBackend::unpersist].

pub fn into_inner(this: Sp<T, D>) -> Option<T>

Returns the content of this Sp, if this Sp is initialized, and is the only reference to its data. When the Sp is initialized, this behaves like Arc::into_inner.

Source

impl<T, D> Sp<T, D>
where T: 'static + ?Sized, D: DB,

pub fn unload(&mut self)

Replace the self.data with an uninitialized lazy value.

Note: if there are multiple outstanding refs to the data in this Sp, e.g. because a clone of this Sp is owned by some larger data structure, then the underlying data won’t actually be dropped until all such Sps go out of scope or have unload called on them.

Warning: If this is the last reference to the Sp, and the Sp is not persisted, dereferencing it after unload may fail, because nothing is keeping the data alive.

Source

impl<T, D> Sp<T, D>
where T: Storable<D>, D: DB,

pub fn serialize_to_node_list(&self) -> TopoSortedNodes

Topologically sort a sub-graph of storage into a sequence of nodes.

This will force and load all nodes in this sub-graph.

pub fn serialize_to_node_list_bounded( &self, raw_size_limit: u64, ) -> Option<TopoSortedNodes>

Topologically sort a sub-graph of storage into a sequence of nodes.

This will force and load all nodes in this sub-graph.

The size limit stops serialization if a specified serialized size limit is overstepped.

Only returns None if a size limit is provided and overstepped