pub struct TopoSortedNodes {
pub nodes: Vec<TopoSortedNode>,
}Expand description
A topologically sorted sub-graph of the storage graph.
Stored as a sequence of nodes, each referencing their children as indices in this sequence. The final entry in this is the root of the sub-graph (which is assumed to have only one root). Each node in the graph should have its children preceding it in the graph, allowing the graph to be restored from this representation in a single in-order iteration.
Fields
nodes: Vec<TopoSortedNode>The individual sorted nodes