RcMap

Struct RcMap 

pub struct RcMap<D: DB = DefaultDB> { /* private fields */ }
Reference count map for tracking charged keys in write and delete costing.

Internally we use ChildRef to ensure that nodes for all keys in the RcMap will be persisted as long a the RcMap itself is.