Array

Struct Array 

pub struct Array<V: Storable<D>, D: DB = DefaultDB>(/* private fields */);
An array built from a MerklePatriciaTrie

Implementations

Source

impl<V: Storable<D>, D: DB> Array<V, D>

pub fn new() -> Self

Construct an empty new array

pub fn new_from_slice(values: &[V]) -> Self

Generates a new Array from a value slice

pub fn len(&self) -> usize

Number of elements in Array.

The elements are stored at indices 0..len().

pub fn get(&self, index: usize) -> Option<&V>

Get element at index. Returns None if index is out of bounds.

pub fn insert(&self, index: usize, value: V) -> Option<Self>

Insert element at index.

Must be an existing index, or returns None. Use push if you want to grow the array.

pub fn push(&self, value: V) -> Self

Appends an element to the end of the array, growing the length by 1.

pub fn into_inner_for_drop(self) -> impl Iterator<Item = V>

Consume internal pointers, returning only the leaves left dangling by this. Used for custom Drop implementations.

pub fn iter(&self) -> ArrayIter<'_, V, D>

Iterate over the elements in the array as Sp<V>s

pub fn iter_deref(&self) -> impl Iterator<Item = &V>

Iterate over the elements in the array as &V references

pub fn is_empty(&self) -> bool

Returns true if empty