pub struct HashMap<K: Serializable + Storable<D>, V: Storable<D>, D: DB = DefaultDB, A: Storable<D> + Annotation<(Sp<K, D>, Sp<V, D>)> = SizeAnn>(/* private fields */);Expand description
A map from key hashes to values
Implementations
Sourceimpl<K: Serializable + Storable<D>, V: Storable<D>, D: DB, A: Storable<D> + Annotation<(Sp<K, D>, Sp<V, D>)>> HashMap<K, V, D, A>
impl<K: Serializable + Storable<D>, V: Storable<D>, D: DB, A: Storable<D> + Annotation<(Sp<K, D>, Sp<V, D>)>> HashMap<K, V, D, A>
Sourcepub fn insert(&self, key: K, value: V) -> Self
pub fn insert(&self, key: K, value: V) -> Self
Insert object value in map, keyed with the hash of object key. Overwrites any preexisting object under the same key
Sourcepub fn contains_key(&self, key: &K) -> bool
pub fn contains_key(&self, key: &K) -> bool
Check if the map contains a key.
Sourcepub fn into_inner_for_drop(self) -> impl Iterator<Item = (Option<K>, Option<V>)>
pub fn into_inner_for_drop(self) -> impl Iterator<Item = (Option<K>, Option<V>)>
Consume internal pointers, returning only the leaves left dangling by this.
Used for custom Drop implementations.