HashSet

Struct HashSet 

pub struct HashSet<V: Storable<D> + Serializable, D: DB = DefaultDB, A: Storable<D> + Annotation<(Sp<V, D>, Sp<(), D>)> = SizeAnn>(pub HashMap<V, (), D, A>);
A set. Uses HashMap under the hood.

Tuple Fields

0: HashMap<V, (), D, A>

Implementations

Source

impl<V: Storable<D> + Serializable, D: DB, A: Storable<D> + Annotation<(Sp<V, D>, Sp<(), D>)>> HashSet<V, D, A>

pub fn new() -> Self

Creates an empty set

pub fn insert(&self, value: V) -> Self

Insert object value. Overwrites any preexisting object under the same value

pub fn remove(&self, value: &V) -> Self

Remove object

pub fn member(&self, value: &V) -> bool

Check if the set contains a value.

pub fn is_subset(&self, other: &HashSet<V, D, A>) -> bool

Check if a HashSet is the subset of another HashSet.

pub fn union(&self, other: &HashSet<V, D, A>) -> HashSet<V, D, A>
where V: Clone,

Union with another set

pub fn iter(&self) -> impl Iterator<Item = Arc<Sp<V, D>>> + '_
where V: Clone,

Iterate over the key value pairs in the hash set

pub fn size(&self) -> usize

Number of elements in the set

pub fn is_empty(&self) -> bool

Returns true if empty

pub fn ann(&self) -> A

Retrieve the annotation on the root of the trie