pub struct Map<K, V: Storable<D>, D: DB = DefaultDB, A: Storable<D> + Annotation<V> = SizeAnn> { /* private fields */ }Expand description
A persistently stored map, guaranteeing O(1) clones and log-time modifications.
Implementations
Sourceimpl<K: Serializable + Deserializable, V: Storable<D>, D: DB, A: Storable<D> + Annotation<V>> Map<K, V, D, A>
impl<K: Serializable + Deserializable, V: Storable<D>, D: DB, A: Storable<D> + Annotation<V>> Map<K, V, D, A>
Sourcepub fn insert(&self, key: K, value: V) -> Self
pub fn insert(&self, key: K, value: V) -> Self
Insert a key-value pair into the map. Must be O(log(|self|)).
Sourcepub fn into_inner_for_drop(self) -> impl Iterator<Item = V>
pub fn into_inner_for_drop(self) -> impl Iterator<Item = V>
Consume internal pointers, returning only the leaves left dangling by this.
Used for custom Drop implementations.
Sourcepub fn iter(
&self,
) -> impl Iterator<Item = (K, Sp<V, D>)> + use<'_, K, V, D, A> + '_
pub fn iter( &self, ) -> impl Iterator<Item = (K, Sp<V, D>)> + use<'_, K, V, D, A> + '_
Iterate over the key-value pairs in the map in a deterministic, but unspecified order.
Sourcepub fn keys(&self) -> impl Iterator<Item = K> + use<'_, K, V, D, A> + '_
pub fn keys(&self) -> impl Iterator<Item = K> + use<'_, K, V, D, A> + '_
Iterator over the keys in the map in a deterministic, but unspecified order.
Sourcepub fn contains_key<Q>(&self, key: &Q) -> bool
pub fn contains_key<Q>(&self, key: &Q) -> bool
Check if the map contains a key. Must be O(log(|self|)).
Sourcepub fn get<Q>(&self, key: &Q) -> Option<&V>where
K: Borrow<Q>,
Q: Serializable,
pub fn get<Q>(&self, key: &Q) -> Option<&V>where
K: Borrow<Q>,
Q: Serializable,
Retrieve the value stored at a key, if applicable. Must be O(log(|self|)).