pub struct Sp<T, D = InMemoryDB>{
pub child_repr: ArenaKey<<D as DB>::Hasher>,
pub arena: Arena<D>,
pub root: ArenaHash<<D as DB>::Hasher>,
/* private fields */
}Expand description
A typed pointer to a value stored in the Arena.
An Sp<T> can be lazily initialized, in which case its internal T value
won’t be loaded until access is attempted.
Fields
child_repr: ArenaKey<<D as DB>::Hasher>This Sp represented as a child node (for easy access)
arena: Arena<D>The arena this Sp points into
root: ArenaHash<<D as DB>::Hasher>The persistent hash of data.
Implementations
Sourceimpl<T, D> Sp<T, D>
Constructors for Sp that take care of Arena ref counting.
impl<T, D> Sp<T, D>
Constructors for Sp that take care of Arena ref counting.
Sourcepub fn into_tracked(&self) -> Sp<T, D>
pub fn into_tracked(&self) -> Sp<T, D>
Converts this Sp into one that is tracked directly, making it possible to lookup by its hash.
This forces the Sp to be considered a reference when used as a child of other Sps, and places it into internal caches, and eventually the database if persisted or a parent is persisted.
Sourceimpl<D> Sp<dyn Any + Send + Sync, D>where
D: DB,
impl<D> Sp<dyn Any + Send + Sync, D>where
D: DB,
Sourcepub fn downcast<T>(&self) -> Option<Sp<T, D>>
pub fn downcast<T>(&self) -> Option<Sp<T, D>>
Downcasts this dynamically typed pointer to a concrete type, if possible.
Sourcepub fn force_downcast<T>(&self) -> Sp<T, D>
pub fn force_downcast<T>(&self) -> Sp<T, D>
Downcasts this dynamically typed pointer to a concrete type, but pushes through the cast regardless of the underlying type.
This will effectively unload the Sp, and construct a new lazy Sp with the same backing data. There is no way of knowing if this will succeed, as the lazy loading will defer failure to a context where a failure panics.
Sourceimpl<T, D> Sp<T, D>
impl<T, D> Sp<T, D>
Sourcepub fn is_lazy(&self) -> bool
pub fn is_lazy(&self) -> bool
Return true iff this Sp is lazy/unforced, i.e. its data has not yet
been loaded.
If the Sp is lazy, it can be forced by dereferencing it.
Sourcepub fn hash(&self) -> ArenaHash<<D as DB>::Hasher>
pub fn hash(&self) -> ArenaHash<<D as DB>::Hasher>
Return hash of self and all children, cached from <T as Storable>::hash().
This is the root key of self, as a content-addressed Merkle node.
Sourcepub fn as_typed_key(&self) -> TypedArenaKey<T, <D as DB>::Hasher>
pub fn as_typed_key(&self) -> TypedArenaKey<T, <D as DB>::Hasher>
Returns the TypedArenaKey representation of this Sp, useful as a
reference to persist.
Sourceimpl<T, D> Sp<T, D>
impl<T, D> Sp<T, D>
Sourcepub fn persist(&mut self)
pub fn persist(&mut self)
Notify the storage back-end to increment the persist count on this object.
See [StorageBackend::persist].
Note: Due to a technicality in past behaviour, this converts the Sp into a Ref
internally. In particular, this means that Sp::as_typed_key will provide different
results before and after persisting. A change to this would be a major breaking
change, and is therefore deferred.
Sourcepub fn unpersist(&self)
pub fn unpersist(&self)
Notify the storage back-end to decrement the persist count on this object.
See [StorageBackend::unpersist].
Sourcepub fn into_inner(this: Sp<T, D>) -> Option<T>
pub fn into_inner(this: Sp<T, D>) -> Option<T>
Returns the content of this Sp, if this Sp is initialized, and is
the only reference to its data. When the Sp is initialized, this
behaves like Arc::into_inner.
Sourceimpl<T, D> Sp<T, D>
impl<T, D> Sp<T, D>
Sourcepub fn unload(&mut self)
pub fn unload(&mut self)
Replace the self.data with an uninitialized lazy value.
Note: if there are multiple outstanding refs to the data in this Sp,
e.g. because a clone of this Sp is owned by some larger data
structure, then the underlying data won’t actually be dropped until all
such Sps go out of scope or have unload called on them.
Warning: If this is the last reference to the Sp, and the Sp is not persisted, dereferencing it after unload may fail, because nothing is keeping the data alive.
Sourceimpl<T, D> Sp<T, D>
impl<T, D> Sp<T, D>
Sourcepub fn serialize_to_node_list(&self) -> TopoSortedNodes
pub fn serialize_to_node_list(&self) -> TopoSortedNodes
Topologically sort a sub-graph of storage into a sequence of nodes.
This will force and load all nodes in this sub-graph.
Sourcepub fn serialize_to_node_list_bounded(
&self,
raw_size_limit: u64,
) -> Option<TopoSortedNodes>
pub fn serialize_to_node_list_bounded( &self, raw_size_limit: u64, ) -> Option<TopoSortedNodes>
Topologically sort a sub-graph of storage into a sequence of nodes.
This will force and load all nodes in this sub-graph.
The size limit stops serialization if a specified serialized size limit is overstepped.
Only returns None if a size limit is provided and overstepped